/*
 *  Copyright: Collin Mulliner <collin[at]mulliner.org>
 *  Web: http://www.mulliner.org/nfc/
 *  
 *  License: GPLv2
 *  
 */

import java.io.DataInputStream;
import java.io.IOException;

public class CommandInterface
{
	private byte[] input;
	private int input_length;
	private int input_size;
	private int commandLength;
	private DataInputStream in;
	private DebugInterface dbgout;
	
	private boolean read()
	{
		int rd = 0;
		
		try {
			if (in.available() > 0) {
				dbgout.debug("read", 2);
				rd = in.read(input, input_size, input_length - input_size);
				dbgout.debug(" got " + String.valueOf(rd) + " bytes", 2);
				if (rd >= 0)
					input_size += rd;
				else
					return false;
			}
			return true;
		} catch (IOException e) {
			return false;
		}
	}
	
	private String getLine()
	{
		int p = 0;
		StringBuffer line = new StringBuffer("");
		
		while (p < input_size) {
			if (input[p] == '\n' || input[p] == '\r') {
				shiftInput(p+1);
				dbgout.debug("line: " + line, 2);
				return line.toString();
			}
			line.append((char)input[p]);
			p++;
		}
		
		return null;
	}
	
	private NFCTagCmd parseInput()
	{
		NFCTagCmd cmd = null;
		
		String line = getLine();
		if (line == null)
			return null;
		if (line.startsWith("ReadTag"))
			cmd = new NFCTagCmd(1);
		else if (line.startsWith("WriteTag")) {
			cmd = new NFCTagCmd(2);
			cmd.setData(input, commandLength);
			shiftInput(commandLength);
		}
		return cmd;
	}
	
	public NFCTagCmd getCommand()
	{
		while (read()) {
			if (commandLength == 0) {
				commandLength = getCmdLength();
				if (commandLength == 0) {
					input_size = 0;
					continue;
				}
			}
			if (input_size > 0)
				dbgout.debug("is " + String.valueOf(input_size) + " cl " + String.valueOf(commandLength) + " ", 2);
			if (input_size < commandLength)
				continue;
			else {
				NFCTagCmd tmp = parseInput();
				commandLength = 0;
				return tmp;
			}
		}
		return null;
	}
	
	private void shiftInput(int num)
	{
		int ns = num;
		for (int i = 0; i < input_length; i++) {
			input[i] = input[ns];
			ns++;
			if (ns >= input_length)
				break;
		}
		input_size -= num;
	}
	
	private int getCmdLength()
	{
		String line = getLine();
		if (line == null)
			return 0;
		try {
			int len = Integer.valueOf(line).intValue();
			return len;
		} catch (NumberFormatException e) {
			return 0;
		}
	}
	
	public CommandInterface(DataInputStream inp, DebugInterface dbg)
	{
		dbgout = dbg;
		in = inp;
		input_length = 1024  * 8 + 512;
		input = new byte[input_length];
		input_size = 0;
		commandLength = 0;
	}
}